@@ -1,24 +1,61 @@
 import base64
 import email.message
 import io
-import json
 
 from datetime import datetime
 from functools import partial
 
 from betamax.utils import body_io, coerce_content
 from betamax.matchers import matcher_registry
+from betamax.serializers import serializer_registry, SerializerProxy
 from requests.models import PreparedRequest, Response
 from requests.packages.urllib3 import HTTPResponse
 from requests.structures import CaseInsensitiveDict
+from requests.status_codes import _codes
 
 
-def serialize_prepared_request(request, method):
+def from_list(value):
+    if isinstance(value, list):
+        return value[0]
+    return value
+
+
+def add_body(r, preserve_exact_body_bytes, body_dict):
+    """Simple function which takes a response or request and coerces the body.
+
+    This function adds either ``'string'`` or ``'base64_string'`` to
+    ``body_dict``. If ``preserve_exact_body_bytes`` is ``True`` then it
+    encodes the body as a base64 string and saves it like that. Otherwise,
+    it saves the plain string.
+
+    :param r: This is either a PreparedRequest instance or a Response
+        instance.
+    :param preserve_exact_body_bytes bool: Either True or False.
+    :param body_dict dict: A dictionary already containing the encoding to be
+        used.
+    """
+    body = getattr(r, 'raw', getattr(r, 'body', None))
+    if hasattr(body, 'read'):
+        body = body.read()
+
+    if not body:
+        body = ''
+
+    if (preserve_exact_body_bytes or
+            'gzip' in r.headers.get('Content-Encoding', '')):
+        body_dict['base64_string'] = base64.b64encode(body).decode()
+    else:
+        body_dict['string'] = coerce_content(body, body_dict['encoding'])
+
+
+def serialize_prepared_request(request, preserve_exact_body_bytes):
     headers = request.headers
+    body = {'encoding': 'utf-8'}
+    add_body(request, preserve_exact_body_bytes, body)
     return {
-        'body': request.body or '',
+        'body': body,
         'headers': dict(
-            (coerce_content(k, 'utf-8'), v) for (k, v) in headers.items()
+            (coerce_content(k, 'utf-8'), [v]) for (k, v) in headers.items()
         ),
         'method': request.method,
         'uri': request.url,
@@ -27,24 +64,28 @@ def serialize_prepared_request(request, method):
 
 def deserialize_prepared_request(serialized):
     p = PreparedRequest()
-    p.body = serialized['body']
-    p.headers = CaseInsensitiveDict(serialized['headers'])
+    body = serialized['body']
+    if isinstance(body, dict):
+        original_body = body.get('string')
+        p.body = original_body or base64.b64decode(
+            body.get('base64_string', '').encode())
+    else:
+        p.body = body
+    h = [(k, from_list(v)) for k, v in serialized['headers'].items()]
+    p.headers = CaseInsensitiveDict(h)
     p.method = serialized['method']
     p.url = serialized['uri']
     return p
 
 
-def serialize_response(response, method):
+def serialize_response(response, preserve_exact_body_bytes):
     body = {'encoding': response.encoding}
-    if response.headers.get('Content-Encoding') == 'gzip':
-        body['base64_string'] = base64.b64encode(response.raw.read()).decode()
-    else:
-        body['string'] = coerce_content(response.content, body['encoding'])
+    add_body(response, preserve_exact_body_bytes, body)
 
     return {
         'body': body,
-        'headers': dict(response.headers),
-        'status_code': response.status_code,
+        'headers': dict((k, [v]) for k, v in response.headers.items()),
+        'status': {'code': response.status_code, 'message': response.reason},
         'url': response.url,
     }
 
@@ -52,15 +93,21 @@ def serialize_response(response, method):
 def deserialize_response(serialized):
     r = Response()
     r.encoding = serialized['body']['encoding']
-    r.headers = CaseInsensitiveDict(serialized['headers'])
+    h = [(k, from_list(v)) for k, v in serialized['headers'].items()]
+    r.headers = CaseInsensitiveDict(h)
     r.url = serialized.get('url', '')
-    r.status_code = serialized['status_code']
+    if 'status' in serialized:
+        r.status_code = serialized['status']['code']
+        r.reason = serialized['status']['message']
+    else:
+        r.status_code = serialized['status_code']
+        r.reason = _codes[r.status_code][0].upper()
     add_urllib3_response(serialized, r)
     return r
 
 
 def add_urllib3_response(serialized, response):
-    if response.headers.get('Content-Encoding') == 'gzip':
+    if 'base64_string' in serialized['body']:
         body = io.BytesIO(
             base64.b64decode(serialized['body']['base64_string'].encode())
         )
@@ -79,55 +126,75 @@ def add_urllib3_response(serialized, response):
 
 def timestamp():
     stamp = datetime.utcnow().isoformat()
-    i = stamp.rindex('.')
-    return stamp[:i]
+    try:
+        i = stamp.rindex('.')
+    except ValueError:
+        return stamp
+    else:
+        return stamp[:i]
+
+
+def _option_from(option, kwargs, defaults):
+    value = kwargs.get(option)
+    if value is None:
+        value = defaults.get(option)
+    return value
 
 
 class Cassette(object):
 
-    """The Cassette object abstracts how requests are saved.
-
-    Example usage::
-
-        c = Cassette('vcr/cassettes/httpbin.json', 'json', 'w+')
-        r = requests.get('https://httpbin.org/get')
-        c.save(r)
-
-    No methods or attributes -- other than ``cassette_name`` -- on this object
-    are considered public or part of the public API. As such they are entirely
-    considered implementation details and subject to change. Using or relying
-    on them is not wise or advised.
-
-    """
-
     default_cassette_options = {
         'record_mode': 'once',
         'match_requests_on': ['method', 'uri'],
         're_record_interval': None,
-        'placeholders': []
+        'placeholders': [],
+        'preserve_exact_body_bytes': False
     }
 
-    def __init__(self, cassette_name, serialize, mode='r', placeholders=None):
-        #: Name of the cassette including path, e.g., "vcr/cassettes/name.json"
+    def __init__(self, cassette_name, serialization_format, **kwargs):
+        #: Short name of the cassette
         self.cassette_name = cassette_name
-        self.serialize_format = serialize
+
         self.serialized = None
+
+        defaults = Cassette.default_cassette_options
+
+        # Determine the record mode
+        self.record_mode = _option_from('record_mode', kwargs, defaults)
+
+        # Retrieve the serializer for this cassette
+        serializer = serializer_registry.get(serialization_format)
+        if serializer is None:
+            raise ValueError(
+                'No serializer registered for {0}'.format(serialization_format)
+                )
+
+        self.serializer = SerializerProxy(serializer, cassette_name)
+
+        # Determine which placeholders to use
+        self.placeholders = kwargs.get('placeholders')
+        if not self.placeholders:
+            self.placeholders = defaults['placeholders']
+
+        # Determine whether to preserve exact body bytes
+        self.preserve_exact_body_bytes = _option_from(
+            'preserve_exact_body_bytes', kwargs, defaults
+            )
+
+        # Initialize the interactions
         self.interactions = []
+
+        # Initialize the match options
         self.match_options = set()
-        self.record_mode = Cassette.default_cassette_options['record_mode']
-        self.placeholders = (placeholders or
-                             Cassette.default_cassette_options['placeholders'])
-        self.fd = open(cassette_name, mode)
+
         self.load_interactions()
+        self.serializer.allow_serialization = self.is_recording()
 
     def clear(self):
-        """Clears out this cassette"""
-        # Start a new list
+        # Clear out the interactions
         self.interactions = []
-        # Save that empty list
-        self.save_cassette()
-        # Reset the file object to the start of the file
-        self.fd.seek(0, 0)
+        # Serialize to the cassette file
+        self._save_cassette()
 
     @property
     def earliest_recorded_date(self):
@@ -138,9 +205,7 @@ class Cassette(object):
         return datetime.now()
 
     def eject(self):
-        """Save the interactions to the cassette and close the file."""
-        self.save_cassette()
-        self.fd.close()
+        self._save_cassette()
 
     def find_match(self, request):
         """Find a matching interaction based on the matchers and request.
@@ -169,9 +234,7 @@ class Cassette(object):
 
     def is_empty(self):
         """Determines if the cassette when loaded was empty."""
-        if self.serialized:
-            return False
-        return True
+        return not self.serialized
 
     def is_recording(self):
         """Returns if the cassette is recording."""
@@ -183,51 +246,44 @@ class Cassette(object):
 
     def load_interactions(self):
         if self.serialized is None:
-            self.load_serialized_data()
+            self.serialized = self.serializer.deserialize()
 
-        if 'http_interactions' not in self.serialized:
-            return
+        interactions = self.serialized.get('http_interactions', [])
+        self.interactions = [Interaction(i) for i in interactions]
 
-        self.interactions = [
-            Interaction(i) for i in self.serialized['http_interactions']
-        ]
         for i in self.interactions:
             i.replace_all(self.placeholders, ('placeholder', 'replace'))
 
-    def load_serialized_data(self):
-        if self.serialized:
-            return
-        try:
-            self.serialized = json.load(self.fd)
-        except ValueError:
-            self.serialized = {}
+    def sanitize_interactions(self):
+        for i in self.interactions:
+            i.replace_all(self.placeholders)
 
     def save_interaction(self, response, request):
         interaction = self.serialize_interaction(response, request)
         self.interactions.append(Interaction(interaction, response))
 
-    def save_cassette(self):
-        self.sanitize_interactions()
-        if 'w' in self.fd.mode or 'r+' in self.fd.mode:
-            json.dump({
-                'http_interactions': [i.json for i in self.interactions],
-                'recorded_with': 'betamax',
-            }, self.fd)
-            # Flush the file so that people can inspect files immediately (if
-            # they so please)
-            self.fd.flush()
-
     def serialize_interaction(self, response, request):
         return {
-            'request': serialize_prepared_request(request,
-                                                  self.serialize_format),
-            'response': serialize_response(response, self.serialize_format),
+            'request': serialize_prepared_request(
+                request,
+                self.preserve_exact_body_bytes
+                ),
+            'response': serialize_response(
+                response,
+                self.preserve_exact_body_bytes
+                ),
             'recorded_at': timestamp(),
         }
 
-    def sanitize_interactions(self):
-        for i in self.interactions:
-            i.replace_all(self.placeholders)
+    # Private methods
+    def _save_cassette(self):
+        self.sanitize_interactions()
+
+        cassette_data = {
+            'http_interactions': [i.json for i in self.interactions],
+            'recorded_with': 'betamax/{version}'
+        }
+        self.serializer.serialize(cassette_data)
 
 
 class Interaction(object):
@@ -287,8 +343,8 @@ class Interaction(object):
         for obj in ('request', 'response'):
             headers = self.json[obj]['headers']
             for k, v in list(headers.items()):
-                if text_to_replace in v:
-                    headers[k] = v.replace(text_to_replace, placeholder)
+                v = from_list(v)
+                headers[k] = v.replace(text_to_replace, placeholder)
 
     def replace_in_body(self, text_to_replace, placeholder):
         body = self.json['request']['body']
