@@ -37,6 +37,17 @@ class BaseMatcher(object):
     def __init__(self):
         if not self.name:
             raise ValueError('Matchers require names')
+        self.on_init()
+
+    def on_init(self):
+        """Method to implement if you wish something to happen in ``__init__``.
+
+        The return value is not checked and this is called at the end of
+        ``__init__``. It is meant to provide the matcher author a way to
+        perform things during initialization of the instance that would
+        otherwise require them to override ``BaseMatcher.__init__``.
+        """
+        return None
 
     def match(self, request, recorded_request):
         """This is a method that must be implemented by the user.
@@ -116,8 +127,28 @@ class URIMatcher(BaseMatcher):
     # Matches based on the uri of the request
     name = 'uri'
 
+    def on_init(self):
+        # Get something we can use to match query strings with
+        self.query_matcher = QueryMatcher().match
+
     def match(self, request, recorded_request):
-        return request.url == recorded_request['uri']
+        queries_match = self.query_matcher(request, recorded_request)
+        request_url, recorded_url = request.url, recorded_request['uri']
+        return self.all_equal(request_url, recorded_url) and queries_match
+
+    def parse(self, uri):
+        parsed = urlparse(uri)
+        return {
+            'scheme': parsed.scheme,
+            'netloc': parsed.netloc,
+            'path': parsed.path,
+            'fragment': parsed.fragment
+            }
+
+    def all_equal(self, new_uri, recorded_uri):
+        new_parsed = self.parse(new_uri)
+        recorded_parsed = self.parse(recorded_uri)
+        return (new_parsed == recorded_parsed)
 
 
 _matchers = [BodyMatcher, HeadersMatcher, HostMatcher, MethodMatcher,
