@@ -1,5 +1,5 @@
 import os
-from betamax import matchers
+from betamax import matchers, serializers
 from betamax.adapter import BetamaxAdapter
 from betamax.configure import Configuration
 from betamax.options import Options
@@ -45,13 +45,13 @@ class Betamax(object):
     """
 
     def __init__(self, session, cassette_library_dir=None,
-                 default_cassette_options={}, adapter_args=None):
+                 default_cassette_options={}):
         #: Store the requests.Session object being wrapped.
         self.session = session
         #: Store the session's original adapters.
         self.http_adapters = session.adapters.copy()
         #: Create a new adapter to replace the existing ones
-        self.betamax_adapter = BetamaxAdapter(**(adapter_args or {}))
+        self.betamax_adapter = BetamaxAdapter(old_adapters=self.http_adapters)
         # We need a configuration instance to make life easier
         self.config = Configuration()
         # Merge the new cassette options with the default ones
@@ -64,8 +64,8 @@ class Betamax(object):
             self.config.cassette_library_dir = cassette_library_dir
 
     def __enter__(self):
-        self.session.mount('http://', self.betamax_adapter)
-        self.session.mount('https://', self.betamax_adapter)
+        for k in self.http_adapters:
+            self.session.mount(k, self.betamax_adapter)
         return self
 
     def __exit__(self, *ex_args):
@@ -114,19 +114,31 @@ class Betamax(object):
         """
         matchers.matcher_registry[matcher_class.name] = matcher_class()
 
+    @staticmethod
+    def register_serializer(serializer_class):
+        """Register a new serializer.
+
+        :param matcher_class: (required), this must sub-class
+            :class:`BaseSerializer <betamax.serializers.BaseSerializer>`
+        """
+        name = serializer_class.name
+        serializers.serializer_registry[name] = serializer_class()
+
     def use_cassette(self, cassette_name, **kwargs):
         """Tell Betamax which cassette you wish to use for the context.
 
         :param str cassette_name: relative name, without the serialization
             format, of the cassette you wish Betamax would use
-        :param str serialize: the format you want Betamax to serialize the
-            request and response data to and from
+        :param str serialize_with: the format you want Betamax to serialize
+            the cassette with
+        :param str serialize: DEPRECATED the format you want Betamax to
+            serialize the request and response data to and from
         """
         def _can_load_cassette(name):
             # If we want to record a cassette we don't care if the file exists
             # yet
             recording = False
-            if kwargs['record'] in ['once', 'all']:
+            if kwargs['record'] in ['once', 'all', 'new_episodes']:
                 recording = True
 
             # Otherwise if we're only replaying responses, we should probably
@@ -134,15 +146,14 @@ class Betamax(object):
             return os.path.exists(name) or recording
 
         kwargs = Options(kwargs)
-        serialize = kwargs['serialize']
+        serialize = kwargs['serialize'] or kwargs['serialize_with']
 
         cassette_name = os.path.join(
             self.config.cassette_library_dir, '{0}.{1}'.format(
                 cassette_name, serialize
             ))
 
-        if (_can_load_cassette(cassette_name) and
-                serialize in ('json', 'yaml')):
+        if _can_load_cassette(cassette_name):
             self.betamax_adapter.load_cassette(cassette_name, serialize,
                                                kwargs)
         else:
